%%% Fitting a gammatone function to a modified atom
function [of,oPa] = gammafit1(y,f,Pa,fs),
N = length(y);
P = 5;
dt = 1/fs;
t = dt:dt:N*dt;
c = 0;
phi = 0;
%%%%%%%%%%%%%%%%%%%%%%%%
J = zeros(N,P-1); % Jacobian
a  = 1/Pa(1);
ct = Pa(2);
cf = Pa(3);
ng = Pa(4);
b = Pa(5);
start = fix(ct*fs);
tp = t-(ct-start/fs);
%%%%% Testting parameters
p = 1;
EarQ = 9.26449;
minBW = 24.7;
%%%%%%%%%%%%%%%%%%%%%%%%%
B = ((cf/EarQ)^p +(minBW)^p)^(1/p);
g1 = -a*(((1-ng)*tp.^(ng-2)).*cos(2*pi*cf*tp) - 2*pi*b*B*(tp.^(ng-1)).*cos(2*pi*cf*tp) - 2*pi*cf*(tp.^(ng-1)).*sin(2*pi*cf*tp)).*exp(-2*pi*b*B*tp);
dB = (1/p)*(p/EarQ *(cf/EarQ)^(p-1))*((cf/EarQ)^p + minBW^p)^(1/p -1);
g2 = a*(tp.^(ng-1)).*(-2*pi*b*dB*tp.*cos(2*pi*cf*tp) - 2*pi*tp.*sin(2*pi*cf*tp)).*exp(-2*pi*b*B*tp);
g3 = a*log(tp).*((tp).^(ng-1)).*exp(-2*pi*b*B*tp).*cos(2*pi*cf*tp); % derivative based on n
g4 = -2*pi*a*B*(tp.^ng).*exp(-2*pi*b*B*tp).*cos(2*pi*cf*tp); % derivative based on b
%%%%%%%%%%%%%%
J(1:start-1,:) = [g1(N-start+2:N)', g2(N-start+2:N)',g3(N-start+2:N)' g4(N-start+2:N)'];
J(start:N,:) = [g1(1:N-start+1)', g2(1:N-start+1)',g3(1:N-start+1)', g4(1:N-start+1)'];
%%% Gradient and Gauss-Newton direction derivation
if svds(J'*J,1)/svds(J'*J,1,0) > 10^12,
    Q = diag([10^-7, 10^-3, 1, 1]);
    J = J*Q;
    hgn = -J'*f;
else
    hgn = -inv(J'*J)*J'*f;
    Q = diag([2, 75, 1, 1]);
    hgn = Q*hgn;
end
%%%%%%%%%%%%%%%%%%
step = zeros(P-1,1);
stepi = [1,1,.1,.1]; %%% step size scales for different parameters
eps = .0001;
fp = f;
for j = 1:10 %%% An iterative update of the parameters    
    for i = 2:P %%% An adaptive step size parameter update        
        dstep = stepi(i-1);
        step(i-1) = step(i-1)+dstep;   
        stop = 0;
        nelt = 0;
        while (stop ~= 1)&&(nelt <= 100)
            nelt = nelt + 1;
            oPa = Pa(:) + [0;step.*hgn];
            if ((fix(oPa(2)*fs) <= 0)||(fix(oPa(2)*fs) > N))
                oPa(2) = oPa(2) - sign(fix(oPa(2)*fs)-1)*N/fs;
            end
            %%%%%%%%%%%%%%%%%%%
            a  = 1/Pa(1);
            ct = oPa(2);
            cf = oPa(3);
            ng = oPa(4);
            b = oPa(5);
            start = fix(ct*fs+.00001);
            tp = t-(ct-start/fs);
            B = ((cf/EarQ)^p +(minBW)^p)^(1/p);
            gc = tp.^(ng-1).*exp(-2*pi*b*B*tp).*cos(2*pi*cf*tp+c*log(tp)+phi);
            oPa(1) = sqrt(sum(gc*gc'));
            if oPa(1) < 10^-20
                oPa = Pa;
                fn = f;
                stop = 1;
            else
                fn = zeros(N,1);
                fn(1:start-1) = gc(N-start+2:N)/oPa(1);
                fn(start:N) = gc(1:N-start+1)/oPa(1);
                ep = (fp-y)'*(fp-y);
                ec = (fn-y)'*(fn-y);
                if (ec<ep)                
                    if (ep-ec<eps)
                        stop = 1;
                    else
                        step(i-1) = step(i-1) + dstep;
                        fp = fn;
                    end
                else
                    if (dstep<eps)&&((ep-ec)<eps),
                        stop = 1;
                        if step(i-1)<.001
                            step(i-1) = 0;
                        end
                    else
                        dstep = .5*dstep;
                        step(i-1) = step(i-1) - dstep;                    
                    end
                end            
            end
            
        end        
    end
end
of = fn;